# frozen_string_literal: true

module Hermes
  module Api
    module Dmo
      # checker Api::Dmo::HermesController avant d'ajouter des methodes
      class PublicHousingOffersController < Hermes::Api::Dmo::ApplicationController
        skip_before_action :verify_authenticity_token, :set_user, :allow_true_user, :check_if_log_as
        skip_after_action :log_action

        def mutate_params
          []
        end 
        
        def index
          acceptable_params *Hermes::Api::Dmo::HousingOffersController.acceptable_params.keys
          regex_params *Hermes::Api::Dmo::HousingOffersController.acceptable_params.keys
          valid_operators *Hermes::Api::Dmo::HousingOffersController.valid_operators

          @query_precision = { published: true, housing_channel_keys: ["dc"], reserved: false }
          aggregation = index_aggregation(target_model: Hermes::HousingOffer, no_acl: true)

          @options = { is_public: true }

          paginate(
            aggregation,
            each_serializer: Hermes::HousingOfferSerializer,
            **@options
          )
        end

        def show
          @options = { is_public: true }
          return render status: :unauthorized, json: { errors: ['Offre reservée'], success: false } if @instance.reserved
          super
        end 

        def set_instance
          @instance = Hermes::HousingOffer.find_by(published: true, id: params[:id], housing_channel_keys: ["dc"])
        end

        %w[create update destroy].each do |cud_method_name|
          define_method(cud_method_name) do
            render_json_error :unprocessable_entity, :not_implemented
          end
        end
      end
    end
  end
end
