module Hermes
  module RexClient
    class UpdateContactTask

      def update_contact(rex_entity, client = Hermes::RexClient::Client.new, yopmail = false, skip_update_groups = false, dont_update_lessor = false, dont_update_entity = false)
        fields = rex_entity['fields']
        possible_roles = []
        possible_roles.push('rdsl') unless dont_update_lessor
        possible_roles.push('cre') unless dont_update_entity

        
        
        return if fields['email'].blank? || !possible_roles.include?(fields['rolecrelocal'].downcase)
        
        client.log_puts "updating #{fields['email']} #{fields}"
        
        if Rails.env.test? || yopmail
          fields['email'] = fields['email'].split('@')[0] + '@yopmail.com'
        end
        
        contact = User.find_by(id_cristal: fields['id_cristal']) unless fields['id_cristal'].blank?
        contact = User.find_by(email: fields['email'].downcase) unless contact 
        
        if !contact
          client.log_puts('Creating new user')
          contact = User.create(user_fields(fields).merge(password: Devise.friendly_token[0,20]))
        else
          client.log_puts('Updating existing user')
          contact.skip_confirmation_notification!
          contact.update(user_fields(fields, contact.login, contact.auth_roles))
        end
        
        unless skip_update_groups
          
          # Verifier qu'il n'y a pas un autre contact rex avec le même id_cristal car le rex peut avec 2 contact avec le même id_cristal 1 pour chaque role
          # dans ce cas on merge les entreprises et bailleur des 2 pour eviter de supprimer des habilitations
          unless Rails.env.test? 
            data = client.search_contacts({'fields.id_cristal' => "|#{fields['id_cristal']}"})
            if data.length > 1
              [
                'id_etablissement_cristal',
                'id_bailleur_cristal',
              ].each do |rex_field_name|
                fields[rex_field_name] = data.reduce([]) do |array, b| 
                  rex_id_cristal = b['fields'][rex_field_name]
                  rex_id_cristal = [] if rex_id_cristal.blank?
                  rex_id_cristal = [rex_id_cristal] if rex_id_cristal.is_a? String 
                  array += rex_id_cristal
                end
              end 
            end 
          end

          update_user_right(fields: fields, client: client, type: 'entity', contact: contact) unless dont_update_entity
          update_user_right(fields: fields, client: client, type: 'lessor', contact: contact) unless dont_update_lessor
        end
      end
      
      def update_user_right(options)
        type = options[:type]
        client = options[:client]
        fields = options[:fields]
        contact = options[:contact]

        case type
        when  'entity'
          fieldname = 'id_etablissement_cristal'
          classname = Hermes::Entity
        when 'lessor'
          fieldname = 'id_bailleur_cristal'
          classname = Hermes::Lessor
        else
          raise 'update_user_right type not found'
        end

        return unless fields[fieldname]

        query = fields[fieldname].is_a?(String) ? fields[fieldname] : {'$in': fields[fieldname]}

        id_cristal_manager_group_ids = classname.where(id_cristal: query).pluck(:manager_group_id).map(&:to_s)
        user_right_group_ids = Hermes::UserRight.where(user_id: contact.id, type: type, right: 'administrator').pluck(:group_id)
        cre_national_group_ids = Hermes::UserRight.where(user_id: contact.id, type: type, right: 'administrator', role: 'cre_national').pluck(:group_id)

        added_ids = id_cristal_manager_group_ids - (user_right_group_ids - cre_national_group_ids )
        removed_ids = (user_right_group_ids + cre_national_group_ids ) - id_cristal_manager_group_ids

        added_user_rights = 0
        removed_user_rights = 0
        updated_user_rights = 0

        user_right_data = {
          user_id: contact.id, 
          type: type, 
          right: 'administrator', 
          origin: 'rex',
          role: fields['rolecrelocal'].blank? ? 'cre' : fields['rolecrelocal'].downcase
        }

        added_ids.each do |manager_group_id|
          next unless e = classname.find_by(manager_group_id: manager_group_id)
          if type == 'entity' && fields['rolecrelocal'].downcase == 'cre'
            e.set(updated_at: DateTime.current, cre_email: contact.email, cre_first_name: fields['prenom'], cre_last_name: fields['nom']) 
          end
          next if cre_national_group_ids.include?(manager_group_id)
          begin
            if ur = Hermes::UserRight.find_by(user_id: contact.id, type: type, "#{type}_id": e.id)
              ur.update(user_right_data.merge("#{type}_id": e.id))
              updated_user_rights += 1
            else 
              Hermes::UserRight.create(user_right_data.merge("#{type}_id": e.id))
              added_user_rights += 1
            end 
          rescue => exception
            puts "error: #{exception}"
          end
        end 

        removed_ids.each do |manager_group_id|
          next unless e = classname.find_by(manager_group_id: manager_group_id)
          if type == 'entity' && fields['rolecrelocal'].downcase == 'cre' && e.cre_email == contact.email
            e.set(updated_at: DateTime.current, cre_email: nil, cre_first_name: nil, cre_last_name: nil)
          end 
          next if cre_national_group_ids.include?(manager_group_id)
          if ur = Hermes::UserRight.find_by(user_right_data.merge("#{type}_id": e.id))
            ur.destroy
            removed_user_rights += 1
          end 
        end 

        client.log_puts "Added #{added_user_rights}, updated #{updated_user_rights} and removed #{removed_user_rights} user_rights"
      end 

      def user_fields(fields, contact_login = nil, prev_auth_roles = [])
        mobileNumber = fields['tel_pro'].blank? ? '1234567890' : fields['tel_pro']
        role = fields['rolecrelocal'].blank? ? 'cre' : fields['rolecrelocal'].downcase
        auth_roles = ((prev_auth_roles  || []) + ['collaborateur offre logement'] + [role]).uniq
        login = contact_login || fields['email'].downcase
        {
          username: login,
          first_name: fields['prenom'],
          last_name: fields['nom'],
          id_cristal: fields['id_cristal'],
          email: fields['email'].downcase,
          login: login,
          auth_roles: auth_roles ,
          phone_number: mobileNumber,
          provider: HERMES_CONFIG['identity_provider_key'],
          create_user_data: {
            gender: 'M',
            firstName: fields['prenom'],
            lastName: fields['nom'],
            email: fields['email'].downcase,
            login: login,
            mobileNumber: mobileNumber,
            roles: auth_roles
          },
        }
      end
    end
  end
end
