require 'rails_helper'
require 'vcr_config'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::RequiredDocumentsController, type: :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers

  let!(:responsible) {create(:user, roles: ['Administrator'])}
  let!(:applicant) {create(:applicant, user: responsible, nationality: "1")}
  let!(:housing_request1) {create(:housing_request, main_applicant_id: applicant.id.to_s)}
  let(:required_document_id) {housing_request1.housing_request_folder.required_document_ids[0]}

  describe 'GET /required_documents/:id/historical_status' do
    let!(:status) {create(:status, object_id: required_document_id)}
    let!(:collaborator) {create(:user, roles: ['collaborateur offre logement'])}
    before :each do
      authenticated_header(request, collaborator)
      get 'historical_status', {id: required_document_id}
    end

    it 'returns status code 200' do
      resp = JSON.parse(response.body)
      expect(resp['code']).to eq(200)
    end
  end
end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::RequiredDocumentsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_REQUIRED_DOCUMENTS'
  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:admin) { create(:user, roles: ['Administrator']) }
  let!(:user) { create(:user) }
  let!(:applicant) {create(:applicant, user: user)}
  let!(:housing_request1) {create(:housing_request, main_applicant_id: applicant.id.to_s)}
  let(:required_document_id) {housing_request1.housing_request_folder.required_document_ids[0]}
  let!(:collaborator) {create(:user, roles: ['collaborateur offre logement'])}
  let!(:superadministrateur) {create(:user, roles: ['superadministrateur'])}

  path '/api/dmo/required_documents/{id}' do
    get 'Retrieves a single required_document by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        let(:id) { 1 }

        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }

        run_test!
      end 
      
      response '200', 'required_document found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let(:id) {required_document_id}

        include_context 'with integration test'
      end
    end
  end

  path '/api/dmo/required_documents/{id}/historical_status' do
    get 'Show history' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      parameter name: :id, in: :path, type: :string
      let(:id) {required_document_id}
      let!(:status) {create(:status, object_id: required_document_id)}

      response '200', 'history' do
        let(:Authorization) { "Bearer #{token_for(collaborator)}" }

        run_test!
      end
    end
  end

end
