require 'rails_helper'

RSpec.describe Hermes::Operation, type: :model do
  context 'default' do
    let!(:email1) {'test@email.com'}
    let!(:notification_key1) {'user.new_reserved_offer'}
    let!(:notification_key2) {'user.create'}
    let!(:pref1) { create(:signup_user_notification_preference, { email: 'tesT@email.com', notification_key: notification_key1, want_to_be_subscribed: true }) }
    let!(:pref2) { create(:signup_user_notification_preference, { email: email1, notification_key: notification_key1, want_to_be_subscribed: false }) }
    let!(:pref3) { create(:signup_user_notification_preference, { email: email1, notification_key: notification_key2, want_to_be_subscribed: false }) }
    describe 'signup_user_notification_preference' do 
      it 'works by defaukt' do 
        pref1.destroy
        pref2.destroy
        pref3.destroy
        user = User.create(
          email: email1,
          password: Devise.friendly_token[0,20]
        )
        expect(user.subscription.subscribed_to_email?(notification_key1)).to eq(true)
        expect(user.subscription.subscribed_to_email?(notification_key2)).to eq(true)
      end 

      it 'create proper subscription' do
        user = User.create(
          email: email1,
          password: Devise.friendly_token[0,20]
        )
        user.reload
        user.subscription.reload
        expect(user.subscription.subscribed_to_email?(notification_key1)).to eq(false)
        expect(user.subscription.subscribed_to_email?(notification_key2)).to eq(false)
        expect(pref1.reload.used_at).to be nil
        expect(pref3.reload.used_at).not_to be nil
        expect(pref2.reload.used_at).not_to be nil
        expect(pref3.reload.used_at).not_to be nil
      end
    end 
  end
end
